% Adaptive Quadratur
% Written by: Simon Erne

clear all
close all

%define constants
tol =1e-4;
maxlevel=12;
%define boundaries for each function
a = [3/2 ; 3/2 ; -5 ; 0 ; 0 ; 0; 0];
b = [4   ; 4   ;  5 ; 1 ; 1 ; 1; pi/2];

% define # of functions
p = 7;

% allocate disk space
Q=zeros(1,p+1);
grid=zeros(1,p+1);

%define function handels, "elementwise" operations in case 'x' is a vector
f1 = @(x) (5*sin(20 ./x)).^2 ./(2*x.^3-x.^2);
  [Q1,grid1] = adaptsimp(f1, a(1), b(1) ,tol , maxlevel);
  
f2 = @(x) min(f1(x),1/2);
  [Q2,grid2] = adaptsimp(f2, a(2), b(2) ,tol , maxlevel);
 
f3 = @(x) 1 ./(1+x.^2);
  [Q3,grid3] = adaptsimp(f3, a(3), b(3) ,tol , maxlevel);
 
f4 = @(x) sqrt(x);
  [Q4,grid4] = adaptsimp(f4, a(4), b(4) ,tol , maxlevel);
 
f5 = @(x) sin(4*pi*x).*exp(-2*x);
  [Q5,grid5] =  adaptsimp(f5, a(5), b(5) ,tol , maxlevel);
 
f6 = @(x) sin(4*pi*x).*exp(-2*x);
  [Q6_1, grid6_1] = adaptsimp(f5, a(6), 0.6 ,tol , maxlevel);
  [Q6_2, grid6_2] = adaptsimp(f5, 0.6, b(6) ,tol , maxlevel);
  grid6 = [grid6_1,grid6_2];

f7 = @(x) (sec(x).^2).*exp(-tan(x).^4);
  [Q7,grid7] = adaptsimp(f7, a(7), b(7), tol, maxlevel);

%concat results into cell array
grid={grid1, grid2, grid3 , grid4, grid5 , grid6, grid7};
F={(f1),(f2),(f3),(f4),(f5),(f6),(f7)};


%plot results all in one plot with subplots
figure
for i=1:p
   %function
   subplot (3,3,i)
   x = linspace(a(i),b(i),1000);
   plot(x,F{i}(x),'b-');
   hold on
   % grid
   plot (grid{i},zeros(1,length(grid{i})), 'r+')
   title (['Funktion & Grid ', num2str(i)])
   ylim([ylim]+[-0.1,0.1])
end

saveas(gcf, 'S4_1 - Adapt Quad.png')

% plot results 
% press space for next plot
figure
for i=1:p
   clf;
   %function
   x = linspace(a(i),b(i),1000);
   plot(x,F{i}(x),'b-');
   hold on
   % grid
   plot (grid{i},zeros(1,length(grid{i})), 'r+')
   title (['Funktion & Grid ', num2str(i)])
   ylim([ylim]+[-0.1,0.1])
   pause;
end

% END OF FILE
